--[[
    统一错误码
    格式：[num] = 错误原因          -- 详细说明
    num: 每个模块最少保证有100个错误码
]]
local logger = require "logger"
local errno =  {
    [100000] = "COMMON_PARM_ERR",                                   -- 通用 参数异常
    [100001] = "COMMON_CONF_ERR",                                   -- 通用 配置异常
    [100002] = "COMMON_SYS_ERR",                                    -- 通用 系统异常(服务器数据或结构异常)
    [100003] = "COMMON_UNKNOWN_CURRENCY",                           -- 通用 未知的货币
    [100004] = "COMMON_UNKNOWN_HERO",                               -- 通用 未知的角色
    [100005] = "COMMON_UNKNOWN_CARD",                               -- 通用 未知的技能卡牌
    [100006] = "COMMON_UNKNOWN_EQUIP",                              -- 通用 未知的装备
    [100007] = "COMMON_UNKNOWN_DEBRIS",                             -- 通用 未知的碎片
    [100008] = "COMMON_UNKNOWN_GOODS",                              -- 通用 未知的物品
    [100009] = "COMMON_AWARD_FORMAT_ERR",                           -- 通用 奖励格式异常
    [100010] = "COMMON_AWARD_RECEIVED",                             -- 通用 奖励已领取
    [100011] = "COMMON_CLICK_TOO_FAST",                             -- 通用 点击过快
    [100012] = "COMMON_BAG_MAIL_FULL",                              -- 通用 背包和邮件满了
    [100013] = "COMMON_ACT_NOT_OPEN",                               -- 通用 活动未开启
    [100014] = "COMMON_AWARD_NOT_UNLOCK",                           -- 通用 奖励未解锁
    [100015] = "COMMON_HAVE_GIFT",                                  -- 通用 礼包已购买
    [100016] = "COMMON_NO_MORE_GIFT",                               -- 通用 没有更多的礼包数量
    [100017] = "COMMON_GIFT_TIME_LIMIT",                            -- 通用 礼包已过期
    [100018] = "COMMON_NOT_AWARD",                                  -- 通用 没有奖励
    [100019] = "COMMON_NOT_AWARD_CONDITION",                        -- 通用 奖励条件不满足
    [100020] = "COMMON_NOT_ROLE_LEVEL_LIMIT",                       -- 通用 玩家等级不足
    [100021] = "COMMON_BUY_LIMIT",                                  -- 通用 达到购买上限

    [101000] = "ROLE_NOT_AWARD",                                    -- 建筑 没有奖励 
    
    [101100] = "ADVENTURE_NOT_BATTLE",                              -- 冒险 没有进行中的冒险 
    [101101] = "ADVENTURE_NOT_PASS",                                -- 冒险 未通关 
    [101102] = "ADVENTURE_IN_BATTLE",                               -- 冒险 有冒险在进行 

    [101200] = "PAYMENT_LIST_ERR",                                  -- 支付 格式错误
    [101201] = "PAYMENT_NUM_ERR",                                   -- 支付 道具数量异常
    [101202] = "PAYMENT_UNKNOWN_ITEM",                              -- 支付 未知的道具
    [101203] = "PAYMENT_NOT_ENOUGH",                                -- 支付 道具不足

    [101300] = "HERO_START_MAX",                                    -- 角色 达到最大突破等级
    [101301] = "HERO_SAME_PAY_ERR",                                 -- 角色 消耗同名卡异常
    [101302] = "HERO_OTHER_PAY_ERR",                                -- 角色 消耗其他卡异常
    [101303] = "HERO_MAX_LEVEL",                                    -- 角色 达到最大等级
    [101304] = "HERO_NOBODY_CAN_UP_STAR",                           -- 角色 没有角色可以升星
    [101305] = "HERO_IN_BATTLE",                                    -- 角色 英雄上阵中
    [101306] = "HERO_SAME_SID",                                     -- 角色 相同的角色被使用
    [101307] = "HERO_FULL",                                         -- 角色 英雄背包已满

    [101400] = "EMBATTLE_NOT_TARGET",                               -- 阵容 未找到目标
    [101401] = "EMBATTLE_SAME_NAME_HERO",                           -- 阵容 上阵同名英雄

    [101500] = "SKILL_CARD_MAX_LEVEL",                              -- 技能 达到最大等级
    [101510] = "SKILL_CARD_START_MAX",                              -- 技能 达到最大突破等级


    [101600] = "EQUIP_START_MAX",                                   -- 装备 达到最大突破等级
    [101601] = "EQUIP_SAME_PAY_ERR",                                -- 装备 消耗同名卡异常
    [101602] = "EQUIP_OTHER_PAY_ERR",                               -- 装备 消耗其他卡异常
    [101603] = "EQUIP_MAX_LEVEL",                                   -- 装备 达到最大等级
    [101604] = "EQUIP_NOBODY_CAN_UP_STAR",                          -- 装备 没有装备可以升星
    [101605] = "EQUIP_IN_BATTLE",                                   -- 装备 英雄上阵中
    [101606] = "EQUIP_SAME_SID",                                    -- 装备 相同的角色被使用
    [101607] = "EQUIP_FULL",                                        -- 装备 装备背包已满
    [101608] = "EQUIP_POS_ERR",                                     -- 装备 装备位置异常
    [101609] = "EQUIP_BE_USED",                                     -- 装备 已被穿戴
    [101610] = "EQUIP_PROFESSION_ERR",                              -- 装备 职业不匹配

    [101700] = "QUEST_NOT_COMPLATE",                                -- 任务 任务未完成

    [101800] = "DRAW_DAILY_TIMES_LIMIT",                            -- 抽奖 每日次数限制
    [101801] = "DRAW_FREE_TIMES_LIMIT",                             -- 抽奖 免费次数不足

    [101900] = "DAILY_DUNGEONS_CANT_PASS",                          -- 日常副本 未通关
    [101901] = "DAILY_DUNGEONS_BATTLE_TIMES_LIMTE",                 -- 日常副本 挑战次数不足
    [101903] = "DAILY_DUNGEONS_NOT_BATTLE",                         -- 日常副本 没有进行中的战斗

    [102000] = "CITY_SKILL_UNLOKC",                                 -- 主城 技能未解锁
    [102001] = "CITY_SKILL_LEVEL_MAX",                              -- 主城 等级上限

    [103001] = "RELIC_CANT_SELECT_BUF",                             -- 遗迹 无法选择buff
    [103002] = "RELIC_NOT_EVENT",                                   -- 遗迹 没有事件
    [103003] = "RELIC_ERR_DIFF",                                    -- 遗迹 无法挑战更高难度
    [103004] = "RELIC_CANT_RESET",                                  -- 遗迹 未到重置时间
    [103005] = "RELIC_UNRESOLVED_EVENTS",                           -- 遗迹 有未处理的事件
    [103006] = "RELIC_CANT_ENTER_NEXT_POS",                         -- 遗迹 无法通行

    [163001] = "PLYAER_DUPLICATION_NAME",                           -- 玩家 名字重复
    [163002] = "PLYAER_NO_PLAYER",                                  -- 玩家 不存在的玩家
    [163003] = "PLYAER_LLLEGAL_NAME",                               -- 玩家 非法角色名
    [163004] = "PLYAER_LLLEGAL_OPERATION",                          -- 玩家 非法操作
    [163005] = "PLYAER_PARM_LIMIT",                                 -- 玩家 参数异常
    [163006] = "PLYAER_ERR_SERVERID",                               -- 玩家 不支持的服务器ID
    [163007] = "PLYAER_OTHER_LOGIN",                                -- 玩家 其它设备正在登录
    [163008] = "PLYAER_MAINTAIN",                                   -- 玩家 正在维护
    [163009] = "PLYAER_FORBID",                                     -- 玩家 被禁用的帐号
    [163010] = "PLYAER_NO_PLAYER",                                  -- 玩家 未创建角色
    [163011] = "PLYAER_ERR_SIGN",                                   -- 玩家 身份验证失败

    [170001] = "SERVER_START_MAINTENANCE",                          -- 服务器 服务器开始维护
    [170002] = "ACCOUNT_LOGGING_ANOTHER_DEVICE"                     -- 服务器 帐号正在其它设备登录
}

local ret = {}
for k, v in pairs(errno) do
    ret[v] = k
end
setmetatable(ret, {
    __index = function(tbl, tag)
        logger.warn(" _________________未定义错误码: %s", tag)
        logger.warn(" _________________错误来源: %s", debug.traceback())
        return 2000000                                -- 未定义的错误码
    end
})
return ret
