--圣天sdk
local httpc = require "http.httpc"
local logger = require "logger"
local cjson = require "cjson"
local stringify = require "stringify"
local md5 = require "md5"

local trace = logger.trace
local warn = logger.warn
local cjson_decode = cjson.decode
local cjson_encode = cjson.encode

local MODULE = {}

local cdkconf = {
  host = "https://yfy-api.t1degames.com",
  loginUrl = "/login/s/gameAuthentication",
  uploadUrl = "/actor/report/",
  forbidden = false,
  key = "fkybt_519d7c1a09",
  secret = "226f965616a3daac306f9cfb2aa72f58",
  gameCode = "FKYBT",
}

local function sign_str(data)
    -- 提取键到一个数组中
    local keys = {}
    for key, _ in pairs(data) do
        table.insert(keys, key)
    end
    table.sort(keys)

    local str = ""
    for i, key in ipairs(keys) do
        if key ~= "sign" and (key ~= "extension" or data[key] ~= "") then
            if str == "" then
                str = string.format("%s=%s", key, data[key])
            else
                str = string.format("%s&%s=%s", str, key, data[key])
            end
        end
    end
    str = str..cdkconf.secret
    -- trace(str)
    return str
end


function MODULE.login_player(args)
    -- Check input arguments
    local account = args.account
    if account == nil then
        warn("invalid account")
        return
    end

    local token = args.token
    if token == nil then
        warn("invalid token")
        return
    end

    local token = args.token
    local host = cdkconf.host
    local url = cdkconf.loginUrl

    local content = {
        app_key = cdkconf.key,
        extension = "",
        fuse_token = token,
        timestamp = os.time(),
        sign = false
    }
    
    local str = string.format("app_key=%s&fuse_token=%s&timestamp=%s%s", 
        content.app_key,
        content.fuse_token,
        content.timestamp,
        cdkconf.secret
    )
    
    content.sign = string.lower(md5.sumhexa(str))
    local error_msg = ""
    -- trace("content:%s", str)
    -- trace("content.sign:%s", content.sign)
    -- if true then
    --     return true
    -- end
    trace("shengtian: login start")
    local ok, statuscode, body = pcall(httpc.post, host, url, content)
    trace("shengtian: login end")
    if ok then
        local ret = cjson_decode(body)
        -- trace("shengtian: login body"..stringify(ret or {}))
        if ret.code == "200" then
            return account == tostring(ret.data.uid)
        else
            error_msg = ret.msg
        end
    else
        error_msg = string.format("Failed to call %s%s, param=%s",
            host,
            url,
            cjson_encode(content))
    end
    warn(error_msg)
end

function MODULE.recharge_sign(data)
    local contnet = {
        game_app_key = cdkconf.key,
        game_code = cdkconf.gameCode,
        fuse_uid = data.fuse_uid,
        order_id = data.order_id,
        product_id = data.product_id,
        product_name = data.product_name,
        price = data.price,
        role_id = data.role_id,
        role_name = data.role_name,
        role_level = data.role_level,
        server_code = data.server_code,
        server_name = data.server_name,
        extension = data.extension,
        timestamp = data.time,
    }
    local str = sign_str(contnet)
    return string.lower(md5.sumhexa(str))
end

function MODULE.upload(uptype, data)
    local host = cdkconf.host
    local url = cdkconf.uploadUrl..cdkconf.gameCode

    local content = {
        time = os.time(),
        uid = data.account or "",
        type = uptype,
        player_role_id = data.uid or "",
        player_role_name = data.name or "",
        player_server_id = data.server or "",
        player_server_name = data.server or  "",
        player_grade = data.level or "",
        player_rein = "0",
        player_power = data.power or "",
        player_sex = "0",
        player_vip_level = data.vip or "0"
    }
    local str = string.format("game_code=%s&time=%s&uid=%s%s", cdkconf.gameCode, content.time, content.uid, cdkconf.secret)
    content.sign = md5.sumhexa(str)
    -- trace("shengtian: upload start")
    local ok, statuscode, body = pcall(httpc.post, host, url, content)
    -- trace("shengtian: upload end")
    local error_msg
    if ok then
        local ret = cjson_decode(body)
        -- trace("shengtian: upload body"..stringify(ret or {}))
        if ret.code == "200" then
            return 
        else
            error_msg = ret.msg or ""
        end
    end
    error_msg = error_msg or string.format("Failed upload %s%s, param=%s", host, url, cjson_encode(content))
    warn(error_msg)
end

-- function MODULE.login_test()
--   local args = {
--     account = "27733215",
--     token = "3ab72984f6ebbf085eaa65c6f0c53534bdce21f2",
--   }
--   MODULE.login_player(args)
-- end

function MODULE.sign_test()
    local data = {
        game_app_key = "fkybt_519d7c1a09",
        game_code = "FKYBT",
        fuse_uid = "27733215",
        order_id = "1f309a30-a171-11ef-b457-eb63430875e1",
        product_id = "20003",
        product_name = "t7002",
        price = "99",
        role_id = "00-7262039605090717696",
        role_name = "role#640",
        role_level = "6",
        server_code = "1",
        server_name = "1",
        extension = "{currency:TWD}",
        timestamp = "1731469269"
    }
    trace(string.lower(md5.sumhexa(sign_str(data)))) 
    trace("9bb6f6cbe7d5a3811808544b8efdedd9") 
end

return MODULE